package org.example;

import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import lombok.extern.slf4j.Slf4j;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.types.StructType;
import org.bson.Document;


@Slf4j
public class SparkDataProcessor {
    public static void main(String[] args) {
        try {

            SparkSession spark = SparkSession.builder()
                    .master("local[*]")
                    .appName("Structural Streaming")
                    .getOrCreate();

            Dataset<Row> fromKafka = spark.readStream()
                                    .format("kafka")
                                    .option("kafka.bootstrap.servers", "127.0.0.1:9092")
                                    .option("subscribe", "product-info-topic")
                                    .option("kafka.consumer.group.id", "localStream")
                                    .option("failOnDataLoss", "false")
                                    .option("startingOffsets", "earliest")
                                    .load();


            StructType schema = Encoders.bean(ProductInfo.class).schema();

            Dataset<Row> dataset = fromKafka.selectExpr("cast(value as String) as message")
                                    .select(functions.from_json(functions.col("message"), schema).as("data"))
                                    .select("data.*")
                                    .repartition(100);

            Dataset<Row> filteredDataset = dataset.filter(functions.col("deliveryStatus").equalTo("Shipped"));

//            log.info("Total rows = " + filteredDataset.count() );

            Dataset<ProductInfo> ds = filteredDataset.as(Encoders.bean(ProductInfo.class));

//            StreamingQuery query1 = filteredDataset.writeStream().format("console").outputMode(OutputMode.Update()).start();

            StreamingQuery query = ds.writeStream().outputMode(OutputMode.Append())
                    .option("checkpointLocation", "checkpointlocaion/streamingjob")
                    .foreach(new ForeachWriter<ProductInfo>() {
                        private MongoClient mongoClient;
                        private MongoDatabase database;
                        private MongoCollection<Document> collection;

                        @Override
                        public boolean open(long partitionId, long epochId) {
                            mongoClient = MongoClients.create("mongodb://localhost:27017");
                            database = mongoClient.getDatabase("UserDB");
                            collection = database.getCollection("products");
                            return true;
                        }

                        @Override
                        public void process(ProductInfo productInfo) {
                            // Create a document with the object data
                            Document document = new Document();
                            document.append("productId", productInfo.getProductId());
                            document.append("productName", productInfo.getProductName());
                            document.append("productPrice", productInfo.getProductPrice());
                            document.append("deliveryStatus", productInfo.getDeliveryStatus());
                            document.append("timestamp", productInfo.getTimestamp());

                            // Insert the document into the collection
                            collection.insertOne(document);
                        }

                        @Override
                        public void close(Throwable errorOrNull) {
                            // Close the MongoDB connection
                            mongoClient.close();
                        }
                    }).start();
            query.awaitTermination();


/*
            StreamingQuery query = ds.writeStream()
                    .outputMode(OutputMode.Append())
                    .option("checkpointLocation", "checkpointlocaion/streamingjob")
                    .foreach(new ForeachWriter<ProductInfo>() {
                        private MongoCollection<Document> productInfoCollection;
                        private MongoClient client;
                        private MongoDatabase mongoDatabase;

                        @Override
                        public boolean open(long partitionId, long epochId) {
                            client = MongoClients.create("mongodb://localhost:27017");
                            mongoDatabase = client.getDatabase("test");
                            productInfoCollection = mongoDatabase.getCollection("product-info");
                            return true;
                        }

                        @Override
                        public void process(ProductInfo value) {
                            Document document = new Document();
                            document.append("productId", value.getProductId());
                            document.append("productName", value.getProductName());
                            document.append("productPrice", value.getProductPrice());
                            document.append("deliveryStatus", value.getDeliveryStatus());
                            document.append("timestamp", value.getTimestamp());

                            productInfoCollection.insertOne(document);
                        }

                        @Override
                        public void close(Throwable errorOrNull) {
                            client.close();
                        }

                    }).start();
*/
            query.awaitTermination();

        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
        }
    }
}
